library values_or_else;

import 'dart:async';

class ValuesOrElse<S, T> extends StreamEventTransformer {

  final Function _alternative;

  const ValuesOrElse(this._alternative);

  Stream<T> bind(Stream<S> stream) {
    var result = new StreamController<T>();
    var empty  = true;
    stream.listen(
      (S value) {
        empty = false;
        result.add(value);
      },
      onError: (error) {
        result.addError(error);
      },
      onDone: () {
        if (empty) {
          this._alternative().listen(
            (S value) {
              result.add(value);
            },
            onError: (error) {
              result.addError(error);
            },
            onDone: () {
              result.close();
            }
          );
        } else {
          result.close();
        }
      }
    );
    return result.stream;
  }

}